@@ -22,32 +22,64 @@
 
 
 #include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <errno.h>
 
-#include "utility.h"
-#include "win.h"
-#include "parser.h"
-#include "backend.h"
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#define MODULE_NAME log
+#include <nALFS.h>
+
+#include "handlers.h"
+#include "nprint.h"
 
 
-char handler_name[] = "log";
-char handler_description[] = "Log";
-char *handler_syntax_versions[] = { "2.0", "3.0", NULL };
-// char *handler_attributes[] = { NULL };
-char *handler_parameters[] = { NULL };
-int handler_action = 0;
+#if HANDLER_SYNTAX_2_0 || HANDLER_SYNTAX_3_0
 
+static const char *log_parameters[] = { NULL };
 
-int handler_main(element_s *el)
+static int log_main(element_s *el)
 {
-	if (el->content != NULL) {
-		Nprint_h("%s", el->content);
-	} else {
-		Nprint_h("");
-	}
-	
+	Nprint_h("%s", el->content ? el->content : "");
+
 	return 0;
 }
+
+#endif /* HANDLER_SYNTAX_2_0 || HANDLER_SYNTAX_3_0 */
+
+
+/*
+ * Handlers' information.
+ */
+
+handler_info_s HANDLER_SYMBOL(info)[] = {
+#if HANDLER_SYNTAX_2_0
+	{
+		.name = "log",
+		.description = "Log",
+		.syntax_version = "2.0",
+		.parameters = log_parameters,
+		.main = log_main,
+		.type = 0,
+		.alloc_data = NULL,
+		.is_action = 0,
+		.priority = 0
+	},
+#endif
+#if HANDLER_SYNTAX_3_0
+	{
+		.name = "log",
+		.description = "Log",
+		.syntax_version = "3.0",
+		.parameters = log_parameters,
+		.main = log_main,
+		.type = 0,
+		.alloc_data = NULL,
+		.is_action = 0,
+		.priority = 0
+	},
+#endif
+	{
+		NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0
+	}
+};
