@@ -31,18 +31,21 @@
 #include <sys/stat.h>
 #include <ctype.h>
 
-#include <libxml/tree.h>
-#include <libxml/parser.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
+#include "bufsize.h"
 #include "find.h"
 #include "utility.h"
 #include "parser.h"
 #include "backend.h"
 #include "win.h"
-#include "nalfs.h"
+#include "nalfs-core.h"
 #include "comm.h"
 #include "handlers.h"
-#include "config.h"
+#include "logfiles.h"
+#include "options.h"
 
 #include "logging.h"
 
@@ -52,33 +55,28 @@
 
 #define MAX_PACKAGE_NAME_AND_VERSION_BUF_SIZE 128
 
-/* Some temporary files. */
-#define FILE_STATE_FILE_1	"/tmp/nALFS.state_file_1.XXXXXX"
-#define FILE_STATE_FILE_2	"/tmp/nALFS.state_file_2.XXXXXX"
-#define FILE_INSTALLED		"/tmp/nALFS.installed_files.XXXXXX"
+#define LIST_OF_FILES		"/tmp/nALFS.list_of_files.XXXXXX"
 
 
 typedef enum state_type_e {
 	STATE_TYPE_UNKNOWN = 0,
 	STATE_IS_TIME_STAMP,
-	STATE_IS_LIST_OF_FILES
 } state_type_e;
 
 
-typedef struct statefile_s {
+typedef struct flist_s {
 	int complete;
 	char *name;
-} statefile_s;
+} flist_s;
 
 
 static time_t time_stamp = -1;
 
-static statefile_s *statefile_new;
-static statefile_s *statefile_current;
+static flist_s *list_of_files;
 
 static element_s *current_package;
 
-static xmlDocPtr xml_doc;
+static logs_t *logs;
 
 
 static INLINE int get_stamp_directory_status(
@@ -437,12 +435,12 @@ void log_handler_action(const char *format, ...)
 	va_list ap;
 
 
-	if (! opt_log_handlers) {
+	if (! *opt_log_handlers) {
 		return;
 	}
 
-	if (xml_doc == NULL) { // No open XML file for logging.
-		Debug_logging("log_handler_action: xml_doc is NULL");
+	if (logs == NULL) { // No open file for logging.
+		Debug_logging("log_handler_action: logs is NULL");
 		return;
 	}
 
@@ -450,28 +448,26 @@ void log_handler_action(const char *format, ...)
 	vsnprintf(string, MAX_ACTION_MSG_LEN, format, ap);
 	va_end(ap);
 
-	xmlNewTextChild(xml_doc->children, NULL,
-		(const xmlChar *)EL_NAME_FOR_ACTION,
-		(const xmlChar *)string);
+	logs_add_handler_action(logs, string);
 }
 
-static void free_statefile(statefile_s **statefile)
+static void free_flist(flist_s **f)
 {
-	if (*statefile == NULL) {
+	if (*f == NULL) {
 		return;
 	}
 
-	Debug_logging("Deleting the state file (%s).", (*statefile)->name);
-	delete_file((*statefile)->name);
+	Debug_logging("Deleting the state file (%s).", (*f)->name);
+	delete_file((*f)->name);
 
-	xfree((*statefile)->name);
-	xfree(*statefile);
-	*statefile= NULL;
+	xfree((*f)->name);
+	xfree(*f);
+	*f = NULL;
 }
 
 
 
-static int collect_files(const char *f, statefile_s **statefile)
+static int collect_files(const char *f, flist_s **flist)
 {
 	int i;
 	char filename[strlen(f) + 1];
@@ -493,83 +489,37 @@ static int collect_files(const char *f, statefile_s **statefile)
 	/* Doing allocation here, in case collecting is stopped
 	 * (so that this temporary file can be deleted).
 	 */
-	*statefile = xmalloc(sizeof *statefile);
-	(*statefile)->name = xstrdup(filename);
-	(*statefile)->complete = 0;
+	*flist = xmalloc(sizeof *flist);
+	(*flist)->name = xstrdup(filename);
+	(*flist)->complete = 0;
 
 
-	if (opt_logging_method == LOG_USING_TWO_FINDS) {
-		char *package_str = alloc_package_string(current_package);
-
-		fprintf(fp, "%s: %s\n",
-			STATE_IS_LIST_OF_FILES_MSG, package_str);
-
-		xfree(package_str);
-	}
-
 	Nprint("Collecting files from %s...",
-		opt_find_base ? opt_find_base : "/");
+		*opt_find_base ? *opt_find_base : "/");
 
-	i = do_collect_files(fp, opt_find_base, opt_find_prunes, time_stamp);
+	i = do_collect_files(fp, *opt_find_base, *opt_find_prunes, time_stamp);
 
 	fclose(fp);
 
 	if (i) {
 		Nprint_warn("Collecting files failed or interrupted.");
-		free_statefile(statefile);
+		free_flist(flist);
 		return -1;
 	}
 
 	Debug_logging("Collecting files successful.");
-	(*statefile)->complete = 1;
+	(*flist)->complete = 1;
 
 	return 0;
 }
 
-static INLINE void create_installed_files_file(
-	const char *current,
-	const char *new,
-	const char *filename)
-{
-	xmlNodePtr node;
-
-
-	node = xmlNewTextChild(xml_doc->children,
-		NULL, (const xmlChar *)EL_NAME_FOR_FILES_ROOT, NULL);
-
-	if (opt_logging_method == LOG_USING_TWO_FINDS) {
-		xmlSetProp(node,
-			(const xmlChar *)"method",
-			(const xmlChar *)"two finds");
-
-		xmlNewTextChild(node, NULL,
-			(const xmlChar *)EL_NAME_FOR_FILES_FIND_ROOT,
-			(const xmlChar *)opt_find_base);
-		xmlNewTextChild(node, NULL,
-			(const xmlChar *)EL_NAME_FOR_FILES_FIND_PRUNES,
-			(const xmlChar *)opt_find_prunes);
-
-		/* TODO: Ugly. */
-		execute_command("sort %s %s | uniq -u | sort -bk8 >> %s",
-			current, new, filename);
-
-	} else if (opt_logging_method == LOG_USING_ONE_FIND) {
-		xmlSetProp(node,
-			(const xmlChar *)"method",
-			(const xmlChar *)"time stamp");
-
-		/* TODO: Ugly II. */
-		execute_command("cp -f %s %s", new, filename);
-	}
-}
-
 static INLINE void send_state(void)
 {
 	/*
 	 * Send a time stamp (if any) to the frontend.
 	 */
 	if (time_stamp != -1) {
-		int data_len;
+		size_t data_len;
 		char *data;
 		char *package_string;
 		char time_string[DATE_FORMAT_LEN + 1];
@@ -592,32 +542,19 @@ static INLINE void send_state(void)
 
 		Debug_logging("Sending the time stamp to the frontend...");
 
-		comm_send_from_memory(
-			BACKEND_CTRL_SOCK,
-			CTRL_SENDING_STATE,
-			data,
-			strlen(data));
+		comm_send_ctrl_msg(BACKEND_CTRL_SOCK,
+			CTRL_SENDING_STATE, "Sending data...");
+		comm_send_from_memory(BACKEND_CTRL_SOCK, data, data_len);
 
 		xfree(data);
 	}
-
-	/*
-	 * Send current state file (if any) to the frontend.
-	 */
-	if (statefile_current && statefile_current->complete) {
-		Nprint("Sending the current state "
-			"file to the frontend...");
-		comm_send_from_file(
-			BACKEND_CTRL_SOCK,
-			CTRL_SENDING_STATE,
-			statefile_current->name);
-	}
 }
 
 static void finish_logging(char *installed_files)
 {
 	int size;
-	xmlChar *ptr;
+	char *ptr;
+	char *package_string;
 
 
 	Debug_logging("Finishing logging...");
@@ -626,16 +563,22 @@ static void finish_logging(char *installed_files)
 	 * Send package log file to the frontend.
 	 */
 
-	xmlDocDumpFormatMemory(xml_doc, &ptr, &size, 1);
-	xmlFreeDoc(xml_doc);
-	xml_doc = NULL;
+	logs_dump_to_memory(logs, &ptr, &size);
 
-	Nprint("Sending the package log to the frontend... ");
-	comm_send_from_memory(
-		BACKEND_CTRL_SOCK,
-		CTRL_SENDING_LOG_FILE,
-		(const char *)ptr,
-		size);
+	logs_free(logs);
+	logs = NULL;
+
+	package_string = alloc_package_string(current_package);
+
+	if (*opt_be_verbose) {
+		Nprint("Sending the package log to the frontend... ");
+	}
+
+	comm_send_ctrl_msg(BACKEND_CTRL_SOCK,
+		CTRL_SENDING_LOG_FILE, "%s", package_string);
+	comm_send_from_memory(BACKEND_CTRL_SOCK, ptr, (size_t)size);
+
+	xfree(package_string);
 
 	xfree(ptr);
 
@@ -644,26 +587,19 @@ static void finish_logging(char *installed_files)
 	 */
 
 	if (installed_files) {
-		Debug_logging("Sending a list of installed "
-			"files (%s)... ", installed_files);
+		Debug_logging("Sending a list of installed files (%s)... ",
+			installed_files);
+
 		comm_send_from_file(
 			BACKEND_CTRL_SOCK,
 			CTRL_SENDING_FILES_FILE,
 			installed_files);
-
-
-		Debug_logging("Deleting a list (%s)...", installed_files);
-		delete_file(installed_files);
-
-		xfree(installed_files);
-
 	} else {
 		send_state();
 	}
 
 	time_stamp = -1;
-	free_statefile(&statefile_current);
-	free_statefile(&statefile_new);
+	free_flist(&list_of_files);
 	current_package = NULL;
 }
 
@@ -678,15 +614,13 @@ static INLINE void log_stopped_time(void)
 	t = time(NULL);
 	strftime(time_str, sizeof time_str, DATE_FORMAT, localtime(&t));
 
-	xmlNewTextChild(xml_doc->children, NULL,
-		(const xmlChar *)"stopped",
-		(const xmlChar *)time_str);
+	logs_add_stopped_time(logs, time_str);
 }
 
 void log_stopped_execution(void)
 {
-	if (xml_doc == NULL) { // No open XML file for logging.
-		Debug_logging("log_stopped_execution: xml_doc is NULL");
+	if (logs == NULL) { // No open file for logging.
+		Debug_logging("log_stopped_execution: logs is NULL");
 		return;
 	}
 
@@ -696,25 +630,34 @@ void log_stopped_execution(void)
 }
 
 
+static INLINE int ask_about_element_status(element_s *el)
+{
+	ctrl_msg_s *message;
+	element_s *profile = get_profile_by_element(el);
+
+	comm_send_ctrl_msg(BACKEND_CTRL_SOCK, CTRL_REQUEST_EL_STATUS,
+		"%s %s %d", profile->name, el->name, el->id);
+
+	while ((message = comm_read_ctrl_message(BACKEND_CTRL_SOCK)) == NULL)
+		/* Wait for the first control message. */ ;
+
+	/* RUN_STATUS_DONE is not 0 anyway, so atoi() will do. */
+	return atoi(message->content);
+	
+}
 
 static INLINE char *stage_two_of_logging_changed_files(void)
 {
-	if (get_element_status(current_package) != RUN_STATUS_DONE) {
+	/* We have to ask the frontend about element's run status,
+	 * as backend can't keep it with all the forking that goes around.
+	 */
+	if (ask_about_element_status(current_package) != RUN_STATUS_DONE) {
 		Debug_logging("Not doing stage two - "
 			"package status is not DONE.");
 		return NULL;
 	}
 
-	if (opt_logging_method == LOG_USING_TWO_FINDS) {
-		Debug_logging("Logging using two finds.");
-
-		if (! statefile_current) {
-			Debug_logging("Not doing stage two - "
-				"no current state file.");
-			return NULL;
-		}
-
-	} else if (opt_logging_method == LOG_USING_ONE_FIND) {
+	if (*opt_logging_method == LOG_USING_ONE_FIND) {
 		Debug_logging("Logging using one find.");
 
 		if (time_stamp == -1) {
@@ -724,22 +667,11 @@ static INLINE char *stage_two_of_logging_changed_files(void)
 		}
 	}
 
-	collect_files(FILE_STATE_FILE_2, &statefile_new);
+	collect_files(LIST_OF_FILES, &list_of_files);
 
-	if (statefile_new) {
-		char *filename = xstrdup(FILE_INSTALLED);
-
-		if (create_temp_file(filename)) {
-			xfree(filename);
-			return NULL;
-		}
-
-		create_installed_files_file(
-			statefile_current ? statefile_current->name : NULL,
-			statefile_new->name,
-			filename);
-
-		return filename;
+	if (list_of_files) {
+		logs_add_installed_files(logs, *opt_find_base, *opt_find_prunes);
+		return list_of_files->name;
 	}
 
 	return NULL;
@@ -750,13 +682,13 @@ static INLINE void end_package_logging(int status)
 	char *installed_files = NULL;
 
 
-	if (opt_logging_method != LOG_OFF) {
+	if (*opt_logging_method != LOG_OFF) {
 		installed_files = stage_two_of_logging_changed_files();
 	} else {
 		Debug_logging("Logging files is off, not entering stage two.");
 	}
 
-	if (opt_stamp_packages) {
+	if (*opt_stamp_packages) {
 		char *name = alloc_package_name(current_package);
 		char *version = alloc_package_version(current_package);
 
@@ -779,17 +711,7 @@ static INLINE void end_package_logging(int status)
 
 static void stage_one_of_logging_changed_files(void)
 {
-	if (opt_logging_method == LOG_USING_TWO_FINDS) {
-		Debug_logging("Logging using two finds.");
-
-		/* Check if the frontend sent us a state file. */
-		if (statefile_current == NULL) {
-			collect_files(FILE_STATE_FILE_1, &statefile_current);
-		} else {
-			Nprint("Using the existing state file.");
-		}
-
-	} else if (opt_logging_method == LOG_USING_ONE_FIND) {
+	if (*opt_logging_method == LOG_USING_ONE_FIND) {
 		Debug_logging("Logging using one find.");
 
 		/* Check if the frontend sent us a time stamp. */
@@ -797,8 +719,8 @@ static void stage_one_of_logging_changed_files(void)
 			Nprint("Creating new time stamp.");
 			time_stamp = time(NULL);
 
-			if (opt_sleep_after_stamp > 0) {
-				sleep(opt_sleep_after_stamp);
+			if (*opt_sleep_after_stamp > 0) {
+				sleep(*opt_sleep_after_stamp);
 			}
 
 		} else {
@@ -807,89 +729,26 @@ static void stage_one_of_logging_changed_files(void)
 	}
 }
 
-static INLINE state_type_e get_state_file_type(const char *line)
-{
-	int silofm_len = strlen(STATE_IS_LIST_OF_FILES_MSG);
-	int sitsm_len = strlen(STATE_IS_TIME_STAMP_MSG);
-
-
-	if (strncmp(line, STATE_IS_LIST_OF_FILES_MSG, silofm_len) == 0) {
-		if (opt_logging_method == LOG_USING_TWO_FINDS) {
-			return STATE_IS_LIST_OF_FILES;
-		}
-
-	} else if (strncmp(line, STATE_IS_TIME_STAMP_MSG, sitsm_len) == 0) {
-		if (opt_logging_method == LOG_USING_ONE_FIND) {
-			return STATE_IS_TIME_STAMP;
-		}
-	}
-
-	return STATE_TYPE_UNKNOWN;
-}
-
 /* Used in the function below. */
 char *strptime(const char *s, const char *format, struct tm *tm);
 
-static INLINE void check_and_store_state(
-	const char *filename,
-	statefile_s **statefile,
-	time_t *t)
+static INLINE void receive_state(void)
 {
-	int should_delete_file = 0;
-	char line[MAX_STATE_FILE_LINE_LEN];
+	char *ptr;
+	size_t size;
+	char *time_str;
 	struct tm broken_down_time;
-	FILE *fp;
 
 
-	if ((fp = fopen(filename, "r")) == NULL) {
-		Nprint_warn("Unable to open \"%s\" for reading: %s",
-				filename, strerror(errno));
-		return;
-	}
+	comm_read_to_memory(BACKEND_CTRL_SOCK, &ptr, &size);
 
-	fgets(line, sizeof line, fp);
-	remove_new_line(line);
+	time_str = strchr(ptr, '\n') + 1;
 
-	switch (get_state_file_type(line)) {
-		case STATE_IS_LIST_OF_FILES:
-			(*statefile) = xmalloc(sizeof **statefile);
+	strptime(time_str, DATE_FORMAT, &broken_down_time);
 
-			(*statefile)->name = xstrdup(filename);
-			(*statefile)->complete = 1;
+	time_stamp = mktime(&broken_down_time);
 
-			Debug_logging("Received state file stored in \"%s\".",
-				(*statefile)->name);
-
-			break;
-
-		case STATE_IS_TIME_STAMP:
-			/* Read a time string line. */
-			fgets(line, sizeof line, fp);
-			remove_new_line(line);
-
-			strptime(line, DATE_FORMAT, &broken_down_time);
-
-			*t = mktime(&broken_down_time);
-
-			Debug_logging("Time stamp stored (%ld).", *t);
-
-			should_delete_file = 1;
-
-			break;
-
-		case STATE_TYPE_UNKNOWN:
-			Debug_logging("Option for state's type is off.");
-
-			should_delete_file = 1;
-
-			break;
-	}
-
-	fclose(fp);
-
-	if (should_delete_file) {
-		delete_file(filename);
-	}
+	Debug_logging("Time stamp stored (%ld).", time_stamp);
 }
 
 static INLINE void request_state(void)
@@ -906,63 +765,27 @@ static INLINE void request_state(void)
 	type = comm_msg_type(message);
 
 	if (type == CTRL_NO_STATE) {
-		Debug_logging("Frontend doesn't have a state file "
-			"to send us.");
+		Debug_logging("Frontend doesn't have a state file.");
 
 	} else if (type == CTRL_SENDING_STATE) {
-		char *f, filename[] = FILE_STATE_FILE_1;
-
-		if (create_temp_file(filename)) {
-			f = NULL; // Reading and discarding all messages.
-		} else {
-			f = filename;
-		}
-
-		Debug_logging("Receiving a state from the frontend...");
-
-		if (comm_read_to_file(BACKEND_CTRL_SOCK, f) != 0) {
-			Nprint_warn("Receiving state file failed.");
-			return;
-		}
-
-		if (f == NULL) {
-			return;
-		}
-
-		check_and_store_state(
-			filename,
-			&statefile_current,
-			&time_stamp);
+		receive_state();
 
 	} else {
 		Nprint_warn("Unexpected control message (%d) received.", type);
 	}
 }
 
-static INLINE xmlDocPtr start_package_logging(element_s *el)
+static INLINE logs_t *start_package_logging(element_s *el)
 {
+	logs_t *l;
+
 	char *name = alloc_package_name(el);
 	char *version = alloc_package_version(el);
-	xmlDocPtr xmld;
 
 
 	current_package = el;
 
-	/*
-	 * Create xmlDoc.
-	 */
-	xmld = xmlNewDoc((const xmlChar *)"1.0");
-
-	xmld->children = xmlNewDocNode(xmld,
-		NULL, (const xmlChar *)EL_NAME_FOR_A_RUN, NULL);
-
-	xmlNewTextChild(xmld->children, NULL,
-		(const xmlChar *)"name",
-		(const xmlChar *)name);
-
-	xmlNewTextChild(xmld->children, NULL,
-		(const xmlChar *)"version",
-		(const xmlChar *)version);
+	l = logs_init_new_run(name, version);
 
 	xfree(name);
 	xfree(version);
@@ -970,13 +793,13 @@ static INLINE xmlDocPtr start_package_logging(element_s *el)
 	// Check if frontend has a state file to send us.
 	request_state();
 
-	if (opt_logging_method != LOG_OFF) {
+	if (*opt_logging_method != LOG_OFF) {
 		stage_one_of_logging_changed_files();
 	} else {
 		Debug_logging("Logging files is off, not entering stage one.");
 	}
 
-	return xmld;
+	return l;
 }
 
 
@@ -985,56 +808,41 @@ void log_end_time(element_s *el, int status)
 {
 	char time_str[DATE_FORMAT_LEN + 1];
 	time_t t;
-	xmlNodePtr node;
 
 
-	if (xml_doc == NULL) { // No open XML file for logging.
-		Debug_logging("log_end_time: xml_doc is NULL");
+	if (logs == NULL) { // No open file for logging.
+		Debug_logging("log_end_time: logs is NULL");
 		return;
 	}
 
 	t = time(NULL);
 	strftime(time_str, sizeof time_str, DATE_FORMAT, localtime(&t));
 
-	node = xmlNewTextChild(xml_doc->children, NULL,
-		(const xmlChar *)el->name,
-		(const xmlChar *)time_str);
-	xmlSetProp(node,
-		(const xmlChar *)"mode",
-		(const xmlChar *)"end");
-	xmlSetProp(node,
-		(const xmlChar *)"status",
-		status ? (const xmlChar *)"failed" : (const xmlChar *)"done");
+	logs_add_end_time(logs, el->name, time_str, status);
 }
 
 void log_start_time(element_s *el)
 {
 	char time_str[DATE_FORMAT_LEN + 1];
 	time_t t;
-	xmlNodePtr node;
 
 
-	if (xml_doc == NULL) { // No open XML file for logging.
-		Debug_logging("log_start_time: xml_doc is NULL");
+	if (logs == NULL) { // No open file for logging.
+		Debug_logging("log_start_time: logs is NULL");
 		return;
 	}
 
 	t = time(NULL);
 	strftime(time_str, sizeof time_str, DATE_FORMAT, localtime(&t));
 
-	node = xmlNewTextChild(xml_doc->children, NULL,
-		(const xmlChar *)el->name,
-		(const xmlChar *)time_str);
-	xmlSetProp(node,
-		(const xmlChar *)"mode",
-		(const xmlChar *)"start");
+	logs_add_start_time(logs, el->name, time_str);
 }
 
 
 
 void start_logging_element(element_s *el)
 {
-	if (!opt_log_backend) { // Logging off.
+	if (!*opt_log_backend) { // Logging off.
 		Debug_logging("start_logging_element: log_backend is off");
 		return;
 	}
@@ -1043,7 +851,7 @@ void start_logging_element(element_s *el)
 
 	if (Is_element_name(el, "package")) {
 		if (package_has_name_and_version(el)) {
-			xml_doc = start_package_logging(el);
+			logs = start_package_logging(el);
 
 		} else {
 			Debug_logging("start_logging_element: no name/version");
@@ -1055,8 +863,8 @@ void start_logging_element(element_s *el)
 
 void end_logging_element(element_s *el, int status)
 {
-	if (xml_doc == NULL) { // No open XML file for logging.
-		Debug_logging("end_logging_element: xml_doc is NULL");
+	if (logs == NULL) { // No open file for logging.
+		Debug_logging("end_logging_element: logs is NULL");
 		return;
 	}
 
